@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | APC                                                                  |
   +----------------------------------------------------------------------+
-  | Copyright (c) 2006-2008 The PHP Group                                |
+  | Copyright (c) 2006-2011 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -29,10 +29,11 @@
 
  */
 
-/* $Id: apc.c 277132 2009-03-14 01:50:57Z shire $ */
+/* $Id: apc.c 307332 2011-01-10 09:01:23Z pajoye $ */
 
 #include "apc.h"
 #include "apc_zend.h"
+#include "apc_cache.h"
 #include "php.h"
 
 #if HAVE_PCRE || HAVE_BUNDLED_PCRE
@@ -49,33 +50,36 @@
 
 /* {{{ memory allocation wrappers */
 
-void* apc_emalloc(size_t n)
+void* apc_emalloc(size_t n TSRMLS_DC)
 {
     void* p = malloc(n);
     if (p == NULL) {
-        apc_eprint("apc_emalloc: malloc failed to allocate %u bytes:", n);
+        apc_error("apc_emalloc: malloc failed to allocate %u bytes:" TSRMLS_CC, n);
+        return NULL;
     }
     return p;
 }
 
-void* apc_erealloc(void* p, size_t n)
+void* apc_erealloc(void* p, size_t n TSRMLS_DC)
 {
     p = realloc(p, n);
     if (p == NULL) {
-        apc_eprint("apc_erealloc: realloc failed to allocate %u bytes:", n);
+        apc_error("apc_erealloc: realloc failed to allocate %u bytes:" TSRMLS_CC, n);
+        return NULL;
     }
     return p;
 }
 
-void apc_efree(void* p)
+void apc_efree(void* p TSRMLS_DC)
 {
     if (p == NULL) {
-        apc_eprint("apc_efree: attempt to free null pointer");
+        apc_error("apc_efree: attempt to free null pointer" TSRMLS_CC);
+        return;
     }
     free(p);
 }
 
-char* apc_estrdup(const char* s)
+char* APC_ALLOC apc_estrdup(const char* s TSRMLS_DC)
 {
     int len;
     char* dup;
@@ -86,23 +90,24 @@ char* apc_estrdup(const char* s)
     len = strlen(s);
     dup = (char*) malloc(len+1);
     if (dup == NULL) {
-        apc_eprint("apc_estrdup: malloc failed to allocate %u bytes:", len+1);
+        apc_error("apc_estrdup: malloc failed to allocate %u bytes:" TSRMLS_CC, len+1);
+        return NULL;
     }
     memcpy(dup, s, len);
     dup[len] = '\0';
     return dup;
 }
 
-void* apc_xstrdup(const char* s, apc_malloc_t f)
+void* APC_ALLOC apc_xstrdup(const char* s, apc_malloc_t f TSRMLS_DC)
 {
-    return s != NULL ? apc_xmemcpy(s, strlen(s)+1, f) : NULL;
+    return s != NULL ? apc_xmemcpy(s, strlen(s)+1, f TSRMLS_CC) : NULL;
 }
 
-void* apc_xmemcpy(const void* p, size_t n, apc_malloc_t f)
+void* APC_ALLOC apc_xmemcpy(const void* p, size_t n, apc_malloc_t f TSRMLS_DC)
 {
     void* q;
 
-    if (p != NULL && (q = f(n)) != NULL) {
+    if (p != NULL && (q = f(n TSRMLS_CC)) != NULL) {
         memcpy(q, p, n);
         return q;
     }
@@ -112,91 +117,36 @@ void* apc_xmemcpy(const void* p, size_t n, apc_malloc_t f)
 /* }}} */
 
 /* {{{ console display functions */
-
-static void my_log(int level, const char* fmt, va_list args)
-{
-    static const char* level_strings[] = {
-        "apc-debug",
-        "apc-notice",
-        "apc-warning",
-        "apc-error"
-    };
-    static const int num_levels = NELEMS(level_strings);
-
-    time_t now;
-    char* buf;          /* for ctime */
-
-	TSRMLS_FETCH();
-
-    fflush(stdout);
-
-    if (level < 0)
-        level = 0;
-    else if (level >= num_levels)
-        level = num_levels-1;
-
-    now = time(0);
-    buf = ctime(&now);  /* TODO: replace with reentrant impl */
-    buf[24] = '\0';
-
-    fprintf(stderr, "[%s] [%s] ", buf, level_strings[level]);
-    vfprintf(stderr, fmt, args);
-
-    if (fmt[0] != '\0' && fmt[strlen(fmt)-1] == ':') {
-        fprintf(stderr, " %s", strerror(errno));
-    }
-
-    if (zend_is_compiling(TSRMLS_C)) {
-        fprintf(stderr, " in %s on line %d.", zend_get_compiled_filename(TSRMLS_C), zend_get_compiled_lineno(TSRMLS_C)); 
-    } else if (zend_is_executing(TSRMLS_C)) {
-        fprintf(stderr, " in %s on line %d.", zend_get_executed_filename(TSRMLS_C), zend_get_executed_lineno(TSRMLS_C)); 
-    }
-    fprintf(stderr, "\n"); 
-
-    if (level == APC_ERROR) {
-        exit(2);
-    }
-}
-
-void apc_eprint(const char* fmt, ...)
-{
-    va_list args;
-    va_start(args, fmt);
-    my_log(APC_ERROR, fmt, args);
-    va_end(args);
-}
-
-void apc_wprint(const char* fmt, ...)
-{
-    va_list args;
-    va_start(args, fmt);
-    my_log(APC_WARNING, fmt, args);
-    va_end(args);
-}
-
-void apc_nprint(const char* fmt, ...)
-{
-    va_list args;
-    va_start(args, fmt);
-    my_log(APC_NOTICE, fmt, args);
-    va_end(args);
-}
-
-void apc_dprint(const char* fmt, ...)
-{
-#ifdef APC_DEBUG
-    va_list args;
-    va_start(args, fmt);
-    my_log(APC_DBG, fmt, args);
-    va_end(args);
+#ifdef ZTS
+# define APC_PRINT_FUNCTION_PARAMETER TSRMLS_C
+#else
+# define APC_PRINT_FUNCTION_PARAMETER format
 #endif
-}
 
+#define APC_PRINT_FUNCTION(name, verbosity)					\
+	void apc_##name(const char *format TSRMLS_DC, ...)			\
+	{									\
+		va_list args;							\
+										\
+		va_start(args, APC_PRINT_FUNCTION_PARAMETER);			\
+		php_verror(NULL, "", verbosity, format, args TSRMLS_CC);	\
+		va_end(args);							\
+	}
+
+APC_PRINT_FUNCTION(error, E_ERROR)
+APC_PRINT_FUNCTION(warning, E_WARNING)
+APC_PRINT_FUNCTION(notice, E_NOTICE)
+
+#ifdef __DEBUG_APC__
+APC_PRINT_FUNCTION(debug, E_NOTICE)
+#else
+void apc_debug(const char *format TSRMLS_DC, ...) {}
+#endif
 /* }}} */
 
 /* {{{ string and text manipulation */
 
-char* apc_append(const char* s, const char* t)
+char* apc_append(const char* s, const char* t TSRMLS_DC)
 {
     int slen;
     int tlen;
@@ -205,14 +155,14 @@ char* apc_append(const char* s, const char* t)
     slen = strlen(s);
     tlen = strlen(t);
 
-    p = (char*) apc_emalloc((slen + tlen + 1) * sizeof(char));
+    p = (char*) apc_emalloc((slen + tlen + 1) * sizeof(char) TSRMLS_CC);
     memcpy(p, s, slen);
     memcpy(p + slen, t, tlen + 1);
 
     return p;
 }
 
-char* apc_substr(const char* s, int start, int length)
+char* apc_substr(const char* s, int start, int length TSRMLS_DC)
 {
     char* substr;
     int src_len = strlen(s);
@@ -231,12 +181,12 @@ char* apc_substr(const char* s, int start, int length)
     }
 
     /* create the substring */
-    substr = apc_xmemcpy(s + start, length + 1, apc_emalloc);
+    substr = apc_xmemcpy(s + start, length + 1, apc_emalloc TSRMLS_CC);
     substr[length] = '\0';
     return substr;
 }
 
-char** apc_tokenize(const char* s, char delim)
+char** apc_tokenize(const char* s, char delim TSRMLS_DC)
 {
     char** tokens;      /* array of tokens, NULL terminated */
     int size;           /* size of tokens array */
@@ -254,7 +204,7 @@ char** apc_tokenize(const char* s, char delim)
     cur  = 0;
     end  = strlen(s) - 1;
 
-    tokens = (char**) apc_emalloc(size * sizeof(char*));
+    tokens = (char**) apc_emalloc(size * sizeof(char*) TSRMLS_CC);
     tokens[n] = NULL;
 
     while (cur <= end) {
@@ -265,11 +215,11 @@ char** apc_tokenize(const char* s, char delim)
         /* resize token array if necessary */
         if (n == size-1) {
             size *= 2;
-            tokens = (char**) apc_erealloc(tokens, size * sizeof(char*));
+            tokens = (char**) apc_erealloc(tokens, size * sizeof(char*) TSRMLS_CC);
         }
 
         /* save the current token */
-        tokens[n] = apc_substr(s, cur, next-cur);
+        tokens[n] = apc_substr(s, cur, next-cur TSRMLS_CC);
 
         tokens[++n] = NULL;
         cur = next + 1;
@@ -280,13 +230,73 @@ char** apc_tokenize(const char* s, char delim)
 
 /* }}} */
 
-/* {{{ apc stat */
+
+/* {{{ apc_win32_restat */
+#ifdef PHP_WIN32
+static int apc_restat(apc_fileinfo_t *fileinfo TSRMLS_DC)
+{
+    HANDLE hFile;
+    BY_HANDLE_FILE_INFORMATION hInfo;
+
+    hFile = CreateFile(fileinfo->fullpath, GENERIC_WRITE, FILE_SHARE_WRITE|FILE_SHARE_READ, NULL, OPEN_ALWAYS, FILE_FLAG_BACKUP_SEMANTICS, NULL);
+
+    if (!hFile) {
+        apc_debug("Cannot create a file HANDLE for %s\n" TSRMLS_CC, fileinfo->fullpath);
+        return -1;
+    }
+
+    if (!GetFileInformationByHandle(hFile, &hInfo)) {
+        apc_debug("Cannot get file information from handle\n" TSRMLS_CC);
+        CloseHandle(hFile);
+        return -1;
+    }
+
+    CloseHandle(hFile);
+
+    fileinfo->st_buf.sb.st_dev = hInfo.dwVolumeSerialNumber;
+    fileinfo->st_buf.sb.st_ino = (((apc_ino_t)(hInfo.nFileIndexHigh) << 32) | (apc_ino_t) hInfo.nFileIndexLow);
+
+    return 0;
+}
+#else
+static int apc_restat(apc_fileinfo_t *fileinfo TSRMLS_DC)
+{
+	return 0;
+}
+#endif
+/* }}} */
+
+/* {{{ apc_search_paths */
 /* similar to php_stream_stat_path */
 #define APC_URL_STAT(wrapper, filename, pstatbuf) \
     ((wrapper)->wops->url_stat((wrapper), (filename), PHP_STREAM_URL_STAT_QUIET, (pstatbuf), NULL TSRMLS_CC))
-int apc_search_paths(const char* filename, const char* path, apc_fileinfo_t* fileinfo)
+
+/* copy out to path_buf if path_for_open isn't the same as filename */
+#define COPY_IF_CHANGED(p) \
+    (char*) (((p) == filename) ? filename : \
+            (strlcpy((char*)fileinfo->path_buf, (p), sizeof(fileinfo->path_buf))) \
+                    ? (fileinfo->path_buf) : NULL)
+
+/* len checks can be skipped here because filename is NUL terminated */
+#define IS_RELATIVE_PATH(filename, len) \
+        ((filename) && (filename[0] == '.' && \
+            (IS_SLASH(filename[1]) || \
+                (filename[1] == '.' && \
+                    IS_SLASH(filename[2])))))
+    
+/* {{{ stupid stringifcation */
+#if DEFAULT_SLASH == '/'
+    #define DEFAULT_SLASH_STRING "/"
+#elif DEFAULT_SLASH == '\\'
+    #define DEFAULT_SLASH_STRING "\\"
+#else
+    #error "Unknown value for DEFAULT_SLASH"
+#endif
+/* }}} */
+
+int apc_search_paths(const char* filename, const char* path, apc_fileinfo_t* fileinfo TSRMLS_DC)
 {
-    char** paths;
+    char** paths = NULL;
     char *exec_fname;
     int exec_fname_length;
     int found = 0;
@@ -294,8 +304,6 @@ int apc_search_paths(const char* filename, const char* path, apc_fileinfo_t* fil
     php_stream_wrapper *wrapper = NULL;
     char *path_for_open = NULL;
 
-    TSRMLS_FETCH();
-
     assert(filename && fileinfo);
 
 
@@ -307,28 +315,42 @@ int apc_search_paths(const char* filename, const char* path, apc_fileinfo_t* fil
 
     if(wrapper != &php_plain_files_wrapper) {
         if(APC_URL_STAT(wrapper, path_for_open, &fileinfo->st_buf) == 0) {
-            strncpy(fileinfo->fullpath, path_for_open, MAXPATHLEN);
-            return 0;
+            fileinfo->fullpath = COPY_IF_CHANGED(path_for_open);
+            return apc_restat(fileinfo TSRMLS_CC);
         }
         return -1; /* cannot stat */
     }
 
     if (IS_ABSOLUTE_PATH(path_for_open, strlen(path_for_open)) && 
             APC_URL_STAT(wrapper, path_for_open, &fileinfo->st_buf) == 0) {
-        strncpy(fileinfo->fullpath, path_for_open, MAXPATHLEN);
-        return 0;
+        fileinfo->fullpath = COPY_IF_CHANGED(path_for_open);
+        return apc_restat(fileinfo TSRMLS_CC);
     }
 
-    paths = apc_tokenize(path, DEFAULT_DIR_SEPARATOR);
-    if (!paths)
-        return -1;
+    if (!IS_RELATIVE_PATH(path_for_open, strlen(path_for_open))) {
+        paths = apc_tokenize(path, DEFAULT_DIR_SEPARATOR TSRMLS_CC);
+        if (!paths)
+            return -1;
 
-    /* for each directory in paths, look for filename inside */
-    for (i = 0; paths[i]; i++) {
-        snprintf(fileinfo->fullpath, sizeof(fileinfo->fullpath), "%s%c%s", paths[i], DEFAULT_SLASH, path_for_open);
-        if (APC_URL_STAT(wrapper, fileinfo->fullpath, &fileinfo->st_buf) == 0) {
-            found = 1;
-            break;
+        /* for each directory in paths, look for filename inside */
+        for (i = 0; paths[i]; i++) {
+            snprintf(fileinfo->path_buf, sizeof(fileinfo->path_buf), "%s%c%s", paths[i], DEFAULT_SLASH, path_for_open);
+            if (APC_URL_STAT(wrapper, fileinfo->path_buf, &fileinfo->st_buf) == 0) {
+                fileinfo->fullpath = (char*) fileinfo->path_buf;
+                found = 1;
+                break;
+            }
+        }
+    } else {
+        /* read cwd and try to fake up fullpath */
+        fileinfo->path_buf[0] = '\0';
+        if(VCWD_GETCWD(fileinfo->path_buf, sizeof(fileinfo->path_buf))) { 
+            strlcat(fileinfo->path_buf, DEFAULT_SLASH_STRING, sizeof(fileinfo->path_buf));
+            strlcat(fileinfo->path_buf, path_for_open, sizeof(fileinfo->path_buf));
+            if (APC_URL_STAT(wrapper, fileinfo->path_buf, &fileinfo->st_buf) == 0) {
+                fileinfo->fullpath = (char*) fileinfo->path_buf;
+                return apc_restat(fileinfo TSRMLS_CC);
+            }
         }
     }
 
@@ -340,23 +362,26 @@ int apc_search_paths(const char* filename, const char* path, apc_fileinfo_t* fil
         while((--exec_fname_length >= 0) && !IS_SLASH(exec_fname[exec_fname_length]));
         if((exec_fname && exec_fname[0] != '[') && exec_fname_length > 0) {
             /* not: [no active file] or no path */
-            memcpy(fileinfo->fullpath, exec_fname, exec_fname_length);
-            fileinfo->fullpath[exec_fname_length] = DEFAULT_SLASH;
-            strlcpy(fileinfo->fullpath +exec_fname_length +1, path_for_open,sizeof(fileinfo->fullpath)-exec_fname_length-1);
-            /* apc_wprint("filename: %s, exec_fname: %s, fileinfo->fullpath: %s", path_for_open, exec_fname, fileinfo->fullpath); */
-            if (APC_URL_STAT(wrapper, fileinfo->fullpath, &fileinfo->st_buf) == 0) {
+            memcpy(fileinfo->path_buf, exec_fname, exec_fname_length);
+            fileinfo->path_buf[exec_fname_length] = DEFAULT_SLASH;
+            strlcpy(fileinfo->path_buf +exec_fname_length +1, path_for_open,sizeof(fileinfo->path_buf)-exec_fname_length-1);
+            /* apc_warning("filename: %s, exec_fname: %s, fileinfo->path_buf: %s" TSRMLS_CC, path_for_open, exec_fname, fileinfo->path_buf); */
+            if (APC_URL_STAT(wrapper, fileinfo->path_buf, &fileinfo->st_buf) == 0) {
+                fileinfo->fullpath = (char*) fileinfo->path_buf;
                 found = 1;
             }
         }
     }
 
-    /* free the value returned by apc_tokenize */
-    for (i = 0; paths[i]; i++) {
-        apc_efree(paths[i]);
+    if(paths) {
+        /* free the value returned by apc_tokenize */
+        for (i = 0; paths[i]; i++) {
+            apc_efree(paths[i] TSRMLS_CC);
+        }
+        apc_efree(paths TSRMLS_CC);
     }
-    apc_efree(paths);
 
-    return found ? 0 : -1;
+    return found ? apc_restat(fileinfo TSRMLS_CC) : -1;
 }
 
 /* }}} */
@@ -385,7 +410,7 @@ typedef struct {
 #define APC_COMPILE_PATTERN(re, match) do {\
     if(match.len > 2) { /* more than just "//" */\
         if (((re) = pcre_get_compiled_regex(match.c, NULL, NULL TSRMLS_CC)) == NULL) {\
-            apc_wprint("apc_regex_compile_array: invalid expression '%s'", match.c); \
+            apc_warning("apc_regex_compile_array: invalid expression '%s'" TSRMLS_CC, match.c); \
             smart_str_free(&match);\
             return NULL;\
         }\
@@ -405,7 +430,7 @@ void* apc_regex_compile_array(char* patterns[] TSRMLS_DC)
     if (!patterns)
         return NULL;
 
-    regs = (apc_regex*) apc_emalloc(sizeof(apc_regex));
+    regs = (apc_regex*) apc_emalloc(sizeof(apc_regex) TSRMLS_CC);
 
     smart_str_appendc(&pmatch, '/');
     smart_str_appendc(&nmatch, '/');
@@ -435,11 +460,11 @@ void* apc_regex_compile_array(char* patterns[] TSRMLS_DC)
     return (void*) regs;
 }
 
-void apc_regex_destroy_array(void* p)
+void apc_regex_destroy_array(void* p TSRMLS_DC)
 {
     if (p != NULL) {
         apc_regex* regs = (apc_regex*) p;
-        apc_efree(regs);
+        apc_efree(regs TSRMLS_CC);
     }
 }
 
@@ -468,7 +493,7 @@ int apc_regex_match_array(void* p, const char* input)
 void* apc_regex_compile_array(char* patterns[] TSRMLS_DC)
 {
     if(patterns && patterns[0] != NULL) {
-        apc_wprint("pcre missing, disabling filters");
+        apc_warning("pcre missing, disabling filters" TSRMLS_CC);
     }
     return NULL;
 }
@@ -618,7 +643,7 @@ HashTable* apc_flip_hash(HashTable *hash) {
         Z_ADDREF_P(data);
         zend_hash_move_forward_ex(hash, &pos);
     }
-    efree(data);
+    zval_ptr_dtor(&data);
 
     return new_hash;
 }
