@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | APC                                                                  |
   +----------------------------------------------------------------------+
-  | Copyright (c) 2006-2008 The PHP Group                                |
+  | Copyright (c) 2006-2011 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -29,7 +29,7 @@
 
  */
 
-/* $Id: apc.h 276125 2009-02-19 01:02:59Z shire $ */
+/* $Id: apc.h 307264 2011-01-08 13:20:20Z gopalv $ */
 
 #ifndef APC_H
 #define APC_H
@@ -61,45 +61,43 @@
 #include "php.h"
 #include "main/php_streams.h"
 
-/* log levels constants (see my_log) */
-enum { APC_DBG, APC_NOTICE, APC_WARNING, APC_ERROR };
-
 /* typedefs for extensible memory allocators */
-typedef void* (*apc_malloc_t)(size_t);
-typedef void  (*apc_free_t)  (void*);
+typedef void* (*apc_malloc_t)(size_t TSRMLS_DC);
+typedef void  (*apc_free_t)  (void * TSRMLS_DC);
 
 /* wrappers for memory allocation routines */
-extern void* apc_emalloc(size_t n);
-extern void* apc_erealloc(void* p, size_t n);
-extern void apc_efree(void* p);
-extern char* apc_estrdup(const char* s);
-extern void* apc_xstrdup(const char* s, apc_malloc_t f);
-extern void* apc_xmemcpy(const void* p, size_t n, apc_malloc_t f);
+extern void* apc_emalloc(size_t n TSRMLS_DC);
+extern void* apc_erealloc(void* p, size_t n TSRMLS_DC);
+extern void apc_efree(void* p TSRMLS_DC);
+extern char* apc_estrdup(const char* s TSRMLS_DC);
+extern void* apc_xstrdup(const char* s, apc_malloc_t f TSRMLS_DC);
+extern void* apc_xmemcpy(const void* p, size_t n, apc_malloc_t f TSRMLS_DC);
 
 /* console display functions */
-extern void apc_eprint(const char* fmt, ...);
-extern void apc_wprint(const char* fmt, ...);
-extern void apc_dprint(const char* fmt, ...);
-extern void apc_nprint(const char* fmt, ...);
+extern void apc_error(const char *format TSRMLS_DC, ...);
+extern void apc_warning(const char *format TSRMLS_DC, ...);
+extern void apc_notice(const char *format TSRMLS_DC, ...);
+extern void apc_debug(const char *format TSRMLS_DC, ...);
 
 /* string and text manipulation */
-extern char* apc_append(const char* s, const char* t);
-extern char* apc_substr(const char* s, int start, int length);
-extern char** apc_tokenize(const char* s, char delim);
+extern char* apc_append(const char* s, const char* t TSRMLS_DC);
+extern char* apc_substr(const char* s, int start, int length TSRMLS_DC);
+extern char** apc_tokenize(const char* s, char delim TSRMLS_DC);
 
 /* filesystem functions */
 
 typedef struct apc_fileinfo_t 
 {
-    char fullpath[MAXPATHLEN+1];
+    char *fullpath;
+    char path_buf[MAXPATHLEN];
     php_stream_statbuf st_buf;
 } apc_fileinfo_t;
 
-extern int apc_search_paths(const char* filename, const char* path, apc_fileinfo_t* fileinfo);
+extern int apc_search_paths(const char* filename, const char* path, apc_fileinfo_t* fileinfo TSRMLS_DC);
 
 /* regular expression wrapper functions */
 extern void* apc_regex_compile_array(char* patterns[] TSRMLS_DC);
-extern void apc_regex_destroy_array(void* p);
+extern void apc_regex_destroy_array(void* p TSRMLS_DC);
 extern int apc_regex_match_array(void* p, const char* input);
 
 /* apc_crc32: returns the CRC-32 checksum of the first len bytes in buf */
@@ -112,7 +110,19 @@ extern HashTable* apc_flip_hash(HashTable *hash);
 #define APC_POSITIVE_MATCH 2
 
 #define apc_time() \
-    (APCG(use_request_time) ? sapi_get_request_time(TSRMLS_C) : time(0));
+    (APCG(use_request_time) ? (time_t) sapi_get_request_time(TSRMLS_C) : time(0));
+
+#if defined(__GNUC__)
+# define APC_UNUSED __attribute__((unused))
+# define APC_USED __attribute__((used))
+# define APC_ALLOC __attribute__((malloc))
+# define APC_HOTSPOT __attribute__((hot))
+#else 
+# define APC_UNUSED
+# define APC_USED
+# define APC_ALLOC 
+# define APC_HOTSPOT 
+#endif
 
 #endif
 
