@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | APC                                                                  |
   +----------------------------------------------------------------------+
-  | Copyright (c) 2006-2008 The PHP Group                                |
+  | Copyright (c) 2006-2011 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -29,7 +29,7 @@
 
  */
 
-/* $Id: apc_php.h 268255 2008-11-04 05:42:11Z rasmus $ */
+/* $Id: apc_php.h 307048 2011-01-03 23:53:17Z kalle $ */
 
 #ifndef APC_PHP_H
 #define APC_PHP_H
@@ -47,6 +47,9 @@
 #include "zend_hash.h"
 #include "zend_extensions.h"
 
+#if ZEND_MODULE_API_NO >= 20100409
+#define ZEND_ENGINE_2_4
+#endif
 #if ZEND_MODULE_API_NO > 20060613
 #define ZEND_ENGINE_2_3
 #endif
