@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | APC                                                                  |
   +----------------------------------------------------------------------+
-  | Copyright (c) 2008 The PHP Group                                     |
+  | Copyright (c) 2006-2011 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -25,7 +25,7 @@
 
  */
 
-/* $Id: apc_pool.h 275212 2009-02-05 08:36:26Z gopalv $ */
+/* $Id: apc_pool.h 307048 2011-01-03 23:53:17Z kalle $ */
 
 #ifndef APC_POOL_H
 #define APC_POOL_H
@@ -33,26 +33,37 @@
 #include "apc.h"
 #include "apc_sma.h"
 
+/* #define APC_POOL_DEBUG 1 */
+
 typedef enum {
     APC_UNPOOL         = 0x0,
     APC_SMALL_POOL     = 0x1,
     APC_MEDIUM_POOL    = 0x2,
     APC_LARGE_POOL     = 0x3,
     APC_POOL_SIZE_MASK = 0x7,   /* waste a bit */
+#if APC_POOL_DEBUG
     APC_POOL_REDZONES  = 0x08,
     APC_POOL_SIZEINFO  = 0x10,
     APC_POOL_OPT_MASK  = 0x18
+#endif
 } apc_pool_type;
 
+#if APC_POOL_DEBUG
 #define APC_POOL_HAS_SIZEINFO(pool) ((pool->type & APC_POOL_SIZEINFO)!=0)
 #define APC_POOL_HAS_REDZONES(pool) ((pool->type & APC_POOL_REDZONES)!=0)
+#else
+/* let gcc optimize away the optional features */
+#define APC_POOL_HAS_SIZEINFO(pool) (0)
+#define APC_POOL_HAS_REDZONES(pool) (0)
+#endif
+
 
 typedef struct _apc_pool apc_pool;
 
-typedef void  (*apc_pcleanup_t)(apc_pool *pool);
+typedef void  (*apc_pcleanup_t)(apc_pool *pool TSRMLS_DC);
 
-typedef void* (*apc_palloc_t)(apc_pool *pool, size_t size);
-typedef void  (*apc_pfree_t) (apc_pool *pool, void* p);
+typedef void* (*apc_palloc_t)(apc_pool *pool, size_t size TSRMLS_DC);
+typedef void  (*apc_pfree_t) (apc_pool *pool, void* p TSRMLS_DC);
 
 typedef void* (*apc_protect_t)  (void *p);
 typedef void* (*apc_unprotect_t)(void *p);
@@ -77,13 +88,13 @@ struct _apc_pool {
     /* apc_realpool and apc_unpool add more here */
 };
 
-#define apc_pool_alloc(pool, size) ((pool)->palloc((pool), (size)))
-#define apc_pool_free (pool, ptr)  ((pool)->pfree((pool), (ptr)))
+#define apc_pool_alloc(pool, size)  ((void *) pool->palloc(pool, size TSRMLS_CC))
+#define apc_pool_free(pool, ptr) 	((void)   pool->pfree (pool, ptr TSRMLS_CC))
 
-#define apc_pool_protect (pool, ptr)  (pool->protect ? \
+#define apc_pool_protect(pool, ptr)  (pool->protect ? \
 										(pool)->protect((ptr)) : (ptr))
 
-#define apc_pool_unprotect (pool, ptr)  (pool->unprotect ? \
+#define apc_pool_unprotect(pool, ptr)  (pool->unprotect ? \
 											(pool)->unprotect((ptr)) : (ptr))
 
 extern void apc_pool_init();
@@ -92,11 +103,12 @@ extern apc_pool* apc_pool_create(apc_pool_type pool_type,
                             apc_malloc_t allocate,
                             apc_free_t deallocate,
 							apc_protect_t protect,
-							apc_unprotect_t unprotect);
+							apc_unprotect_t unprotect
+							TSRMLS_DC);
 
-extern void apc_pool_destroy(apc_pool* pool);
+extern void apc_pool_destroy(apc_pool* pool TSRMLS_DC);
 
-extern void* apc_pmemcpy(const void* p, size_t n, apc_pool* pool);
-extern void* apc_pstrdup(const char* s, apc_pool* pool);
+extern void* apc_pmemcpy(const void* p, size_t n, apc_pool* pool TSRMLS_DC);
+extern void* apc_pstrdup(const char* s, apc_pool* pool TSRMLS_DC);
 
 #endif
