@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | APC                                                                  |
   +----------------------------------------------------------------------+
-  | Copyright (c) 2007-2008 The PHP Group                                |
+  | Copyright (c) 2006-2011 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -17,7 +17,7 @@
 
  */
 
-/* $Id: apc_spin.c 268255 2008-11-04 05:42:11Z rasmus $ */
+/* $Id: apc_spin.c 307048 2011-01-03 23:53:17Z kalle $ */
 
 #include "apc_spin.h"
 
@@ -34,7 +34,7 @@ void apc_slock_destroy(slock_t *lock)
     return;
 }
 
-void apc_slock_lock(slock_t *lock)
+void apc_slock_lock(slock_t *lock TSRMLS_DC)
 {
     S_LOCK(lock);
 }
