@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | APC                                                                  |
   +----------------------------------------------------------------------+
-  | Copyright (c) 2006-2008 The PHP Group                                |
+  | Copyright (c) 2006-2011 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -25,7 +25,7 @@
 
  */
 
-/* $Id: apc_zend.h 286799 2009-08-04 11:27:29Z gopalv $ */
+/* $Id: apc_zend.h 307048 2011-01-03 23:53:17Z kalle $ */
 
 #ifndef APC_ZEND_H
 #define APC_ZEND_H
@@ -76,8 +76,8 @@
 #endif
 
 
-extern void* apc_php_malloc(size_t n);
-extern void apc_php_free(void* p);
+extern void* apc_php_malloc(size_t n TSRMLS_DC);
+extern void apc_php_free(void* p TSRMLS_DC);
 
 extern void apc_zend_init(TSRMLS_D);
 extern void apc_zend_shutdown(TSRMLS_D);
@@ -128,13 +128,19 @@ static inline int _apc_opcode_handler_decode(zend_op *opline)
         _UNUSED_CODE, /* 15             */
         _CV_CODE      /* 16 = IS_CV     */
     };
+#ifdef ZEND_ENGINE_2_4
+    return (opline->opcode * 25) + (apc_vm_decode[opline->op1_type] * 5) + apc_vm_decode[opline->op2_type];
+#else
     return (opline->opcode * 25) + (apc_vm_decode[opline->op1.op_type] * 5) + apc_vm_decode[opline->op2.op_type];
+#endif
 }
 
 # define APC_ZEND_OPLINE                    zend_op *opline = execute_data->opline;
 # define APC_OPCODE_HANDLER_DECODE(opline)  _apc_opcode_handler_decode(opline)
 # if PHP_MAJOR_VERSION >= 6
 #  define APC_OPCODE_HANDLER_COUNT          ((25 * 152) + 1)
+# elif defined(ZEND_ENGINE_2_4)
+#  define APC_OPCODE_HANDLER_COUNT          ((25 * 157) + 1) /* 3 new opcodes in 5.4? - separate, bind_trais, add_trait */
 # elif PHP_MAJOR_VERSION >= 5 && PHP_MINOR_VERSION >= 3
 #  define APC_OPCODE_HANDLER_COUNT          ((25 * 154) + 1) /* 3 new opcodes in 5.3 - unused, lambda, jmp_set */
 # else
@@ -153,9 +159,25 @@ static inline int _apc_opcode_handler_decode(zend_op *opline)
 #define ZEND_FASTCALL
 #endif
 
+/* Added in ZE2.3.0 */
+#ifndef zend_parse_parameters_none
+# define zend_parse_parameters_none() zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "")
+#endif
+
 
 #endif  /* APC_OPCODE_OVERRIDE */
 
+#ifdef ZEND_ENGINE_2_4
+# define ZEND_CE_FILENAME(ce)			(ce)->info.user.filename
+# define ZEND_CE_DOC_COMMENT(ce)        (ce)->info.user.doc_comment
+# define ZEND_CE_DOC_COMMENT_LEN(ce)	(ce)->info.user.doc_comment_len
+# define ZEND_CE_BUILTIN_FUNCTIONS(ce)  (ce)->info.internal.builtin_functions
+#else
+# define ZEND_CE_FILENAME(ce)			(ce)->filename
+# define ZEND_CE_DOC_COMMENT(ce)        (ce)->doc_comment
+# define ZEND_CE_DOC_COMMENT_LEN(ce)	(ce)->doc_comment_len
+# define ZEND_CE_BUILTIN_FUNCTIONS(ce)  (ce)->builtin_functions
+#endif
 
 #endif  /* APC_ZEND_H */
 
