@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | APC                                                                  |
   +----------------------------------------------------------------------+
-  | Copyright (c) 2007-2008 The PHP Group                                |
+  | Copyright (c) 2006-2011 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -19,7 +19,7 @@
 
  */
 
-/* $Id: pgsql_s_lock.h 268255 2008-11-04 05:42:11Z rasmus $ */
+/* $Id: pgsql_s_lock.h 307048 2011-01-03 23:53:17Z kalle $ */
 
 /*-------------------------------------------------------------------------
  *
@@ -884,7 +884,7 @@ extern int	tas_sema(volatile slock_t *lock);
 #define S_LOCK(lock) \
 	do { \
 		if (TAS(lock)) \
-			s_lock((lock), __FILE__, __LINE__); \
+			s_lock((lock), __FILE__, __LINE__ TSRMLS_CC); \
 	} while (0)
 #endif	 /* S_LOCK */
 
@@ -915,7 +915,7 @@ extern int	tas(volatile slock_t *lock);		/* in port/.../tas.s, or
 /*
  * Platform-independent out-of-line support routines
  */
-extern void s_lock(volatile slock_t *lock, const char *file, int line);
+extern void s_lock(volatile slock_t *lock, const char *file, int line TSRMLS_DC);
 
 /* Support for dynamic adjustment of spins_per_delay */
 #define DEFAULT_SPINS_PER_DELAY  100
